import math
import re

from api.database import *
from api.miscellaneous import *


FEEDBACK_EMAIL_MAX_LENGTH = 128
FEEDBACK_EMAIL_REGEX = "\\S+@\\S+\\.\\S+"
FEEDBACK_EMAIL_PATTERN = re.compile(FEEDBACK_EMAIL_REGEX)
FEEDBACK_TEXT_MAX_LENGTH = 16384

FEEDBACK_MIN_PAGE_SIZE = 10
FEEDBACK_MAX_PAGE_SIZE = 500


_SQL_INSERT_FEEDBACK = PreparedStatement(f"""
INSERT INTO "feedback" ("email", "text")
VALUES (?, ?)
""")


def _feedback_entry_db_to_json(feedback_entry_db: DbResultRow):
    feedback_entry_json = {
        "id": feedback_entry_db["id"],
        "time_created": feedback_entry_db["time_created"].strftime(API_DATETIME_FORMAT),
        "text": feedback_entry_db["text"]
    }
    if feedback_entry_db["email"] is not None:
        feedback_entry_json["email"] = feedback_entry_db["email"]
    return feedback_entry_json


def get_feedback_entries(
        entries_per_page: int,
        page: int) -> tuple[int, JsonTypes]:
    if entries_per_page < FEEDBACK_MIN_PAGE_SIZE or entries_per_page > FEEDBACK_MAX_PAGE_SIZE or page < 0:
        raise ValueError("Invalid page size or page number")  # pragma: no cover
    
    row_count_set, entries_db = db_pool.execute_read_statements_in_transaction(
        (f"""
        SELECT COUNT(*) AS "count" FROM "feedback"
        """, []),
        (f"""
        SELECT * FROM "feedback"
        ORDER BY "time_created" DESC, "id" DESC
        LIMIT {entries_per_page} OFFSET {entries_per_page * page}
        """, [])
    )
    return (math.ceil(row_count_set[0]["count"] / entries_per_page),
            list(map(_feedback_entry_db_to_json, entries_db)))


def put_feedback(email: str or None, text: str):
    db_pool.execute_write_transaction(lambda trans: trans.execute_statement_and_commit(
        _SQL_INSERT_FEEDBACK, email, text
    ))
