import re
from typing import TypeVar


DB_ID_PATTERN = re.compile("[a-zA-Z0-9_]{1,100}")


def db_ensure_valid_id(db_id: str):
    if DB_ID_PATTERN.fullmatch(db_id) is None:
        raise ValueError(f"Invalid db id: {db_id}")  # pragma: no cover
    return db_id


DataType = TypeVar("DataType")


def db_collect_id_sorted_data(data_list_db: list, data_list_i: int, id_column: str, id: int, mapping_func=lambda x: x):
    """
    
    :param data_list_db: Must be sorted by id_column ascending
    :param data_list_i:
    :param id_column:
    :param id:
    :param mapping_func:
    :return:
    """
    collected_data = []
    while data_list_i < len(data_list_db):
        column_id = data_list_db[data_list_i][id_column]
        if column_id is not None and column_id < id:
            data_list_i += 1
            continue
        if column_id is None or column_id > id:
            break
        collected_data.append(mapping_func(data_list_db[data_list_i]))
        data_list_i += 1
    return data_list_i, collected_data


def db_collect_unsorted_data(data_list_db: list[DataType], id_column: str, id: int) -> list[DataType]:
    collected_data: list[DataType] = []
    for data_db in data_list_db:
        if data_db[id_column] == id:
            collected_data.append(data_db)
    return collected_data


def db_group_data_by_id(data_list_db: list[DataType], id_column: str) -> dict[int, list[DataType]]:
    result_dict: dict[int, list] = {}
    for data in data_list_db:
        id = data[id_column]
        if id not in result_dict:
            result_dict[id] = []
        result_dict[id].append(data)
    return result_dict
